import { useLazyQuery } from '@apollo/client'
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react'
import { StyleProp, StyleSheet, TextInput, View, ViewStyle } from 'react-native'
import { MentionInput, MentionSuggestionsProps, Suggestion } from 'react-native-controlled-mentions'

import { useSafeDispatch } from '@holi/core/helpers'
import { usePortals } from '@holi/core/providers/PortalProvider'
import { UserByNameResponse, userByNameQuery } from '@holi/core/screens/userprofile/queries'
import type { User } from '@holi/core/screens/userprofile/types'
import HoliText from '@holi/ui/components/atoms/HoliText'
import HoliRichTextInputSuggestions from '@holi/ui/components/organisms/HoliRichTextInput/Suggestions'
import { REGEX_URL, REGEX_USER_MENTIONS_WITH_NAME } from '@holi/ui/helper/richText'
import { useDebounce } from '@holi/ui/helper/useDebounce'
import { HoliTheme, useTheme } from '@holi/ui/styles/theme'

export type HoliRichTextInputProps = {
  initalValue?: string
  placeholder?: string
  autofocus?: boolean
  minSearchChars?: number
  maxSuggestionOptions?: number
  debounceDelay?: number
  onChange?: (text: string) => void
  onFocus?: (event: FocusEvent) => void
  onBlur?: (event: FocusEvent) => void
  onSelectSuggestion?: (suggestion: User) => void
  style?: StyleProp<ViewStyle>
  inputStyle?: StyleProp<ViewStyle>
  testID?: string
  showCharacterCount?: boolean
  maxNoOfCharacters?: number
  inputAccessoryViewID?: string
  placeholderTextColor?: string
}

const HoliRichTextInput: React.FC<HoliRichTextInputProps> = ({
  initalValue = '',
  placeholder = '...',
  autofocus = false,
  minSearchChars = 2,
  maxSuggestionOptions = 5,
  debounceDelay = 300,
  inputAccessoryViewID,
  placeholderTextColor,
  onChange,
  onFocus = () => null,
  onBlur = () => null,
  ...props
}) => {
  const { theme } = useTheme()
  const styles = useMemo(() => getStyles(theme), [theme])

  const inputRef = useRef<TextInput>(null)
  const [value, setValue] = useState(initalValue)
  const [suggestions, setSuggestions] = useState<User[]>([])
  const setSafeSuggestions = useSafeDispatch(setSuggestions)

  const [findUsersByName] = useLazyQuery<UserByNameResponse>(userByNameQuery)
  const { teleport } = usePortals()

  const INITIAL_KEYWORD = ''
  const [keyword, setKeyword] = useState(initalValue)
  const setSafeKeyword = useSafeDispatch(setKeyword)
  const [debouncedKeyword, setDebouncedKeyword] = useState<string>(INITIAL_KEYWORD)
  const setKeywordDebounced = useDebounce(useSafeDispatch(setDebouncedKeyword), debounceDelay)

  const onChooseOptionRef = useRef<(suggestion: Suggestion) => void | null>(() => null)

  const [characterCount, setCharacterCount] = useState(0)
  const [disabled] = useState(false)

  // on mount / unmount
  useEffect(() => {
    // Stop the invocation of the debounced function after unmounting
    return () => setKeywordDebounced.cancel()
  }, [setKeywordDebounced])

  // on value change
  useEffect(() => {
    setKeywordDebounced(keyword)
  }, [keyword, setKeywordDebounced])

  const findUserMapped = useCallback(
    (substring: string): Promise<User[]> => {
      return new Promise((resolve, reject) => {
        findUsersByName({
          variables: {
            substring,
            offset: 0,
            limit: maxSuggestionOptions,
          },
        })
          .then((response) => resolve(response?.data?.userByName?.data?.slice(0, maxSuggestionOptions) ?? []))
          .catch(reject)
      })
    },
    [maxSuggestionOptions, findUsersByName]
  )

  // on debounced value change
  useEffect(() => {
    if (debouncedKeyword === INITIAL_KEYWORD) return setSafeSuggestions([])
    if (!debouncedKeyword || debouncedKeyword.length < minSearchChars) return setSafeSuggestions([])
    findUserMapped(debouncedKeyword).then(setSafeSuggestions)
  }, [debouncedKeyword, minSearchChars, findUserMapped, setSafeSuggestions])

  // on suggestions change
  useEffect(
    () => {
      if (!suggestions.length) return teleport('suggestions', null)

      const element = (
        <HoliRichTextInputSuggestions
          options={suggestions}
          onChooseSuggestion={(user: User) => {
            onChooseOptionRef.current({ id: user.id, name: user.fullName })
            props.onSelectSuggestion?.(user)
            inputRef?.current?.focus()
          }}
          testID={props.testID ? `${props.testID}-suggestions` : 'richtext-input-suggestions'}
        />
      )
      teleport('suggestions', element)
    },
    // `teleport` in the dependencies list causes endless loop
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [suggestions]
  )

  const onInputChange = useCallback(
    (text: string) => {
      const maxChars = props.maxNoOfCharacters || Infinity
      const currentCharacterCount = text.length

      // Check if the new input is within the character limit
      if (currentCharacterCount <= maxChars) {
        setValue(text)
        setCharacterCount(currentCharacterCount)
        const holiFormattedString = String(text).replace(REGEX_USER_MENTIONS_WITH_NAME, '@$1')
        onChange?.(holiFormattedString)
      }
    },
    [onChange, setValue, setCharacterCount, props.maxNoOfCharacters]
  )

  const renderUserSuggestions = useCallback(
    ({ keyword, onSuggestionPress }: MentionSuggestionsProps) => {
      requestAnimationFrame(() => setSafeKeyword(!keyword ? '' : keyword))
      onChooseOptionRef.current = onSuggestionPress
    },
    [setSafeKeyword]
  )

  const suggestionConfig = []
  suggestionConfig.push({
    trigger: '@',
    renderSuggestions: renderUserSuggestions,
    textStyle: styles.inlineMention,
  })

  // TODO: for later usage
  // suggestionConfig.push({
  //   trigger: '#',
  //   renderSuggestions: renderHashtagSuggestions,
  //   textStyle: styles.inlineHashtag,
  // })

  // Nice to have: url highlighting
  suggestionConfig.push({
    pattern: REGEX_URL,
    textStyle: styles.inlineUrl,
  })

  const CharactersCount = () => {
    return (
      <View style={styles.charactorCountContainer}>
        {props.showCharacterCount && (
          <HoliText size="extrasmall" style={styles.charactorCountText}>
            {characterCount} / {props.maxNoOfCharacters}
          </HoliText>
        )}
      </View>
    )
  }

  return (
    <View style={props.style} testID={props.testID || 'richtext-component'}>
      <MentionInput
        inputRef={inputRef}
        value={value}
        autoFocus={autofocus}
        onChange={onInputChange}
        onFocus={onFocus}
        onBlur={onBlur}
        partTypes={suggestionConfig}
        placeholder={placeholder}
        containerStyle={[styles.mentionsStyles, props.style]}
        style={styles.input}
        placeholderTextColor={placeholderTextColor || (theme.dark ? theme.colors.foreground10 : undefined)}
        editable={!disabled}
        maxLength={props.maxNoOfCharacters}
        testID={(props.testID || 'richtext-component') + '-input'}
        inputAccessoryViewID={inputAccessoryViewID}
      />
      <CharactersCount />
    </View>
  )
}

export default HoliRichTextInput

const getStyles = (theme: HoliTheme) =>
  StyleSheet.create({
    mentionsStyles: {
      flexDirection: 'column-reverse' /* input on top and suggestions below */,
      zIndex: 1,
      alignItems: 'flex-start',
      justifyContent: 'flex-end',
    },
    input: {
      width: '100%',
      paddingVertical: 10,
      color: theme.colors.foreground30,
      fontSize: 16,
    },
    inlineMention: {
      color: theme.colors.brandColorB30,
      fontWeight: '600',
    },
    inlineHashtag: {
      color: theme.colors.foreground20,
    },
    inlineUrl: {
      color: theme.colors.foreground20,
    },
    charactorCountText: {
      color: theme.colors.foreground20,
    },
    charactorCountContainer: {
      flex: 1,
      alignItems: 'flex-end',
      marginTop: 6,
    },
  })
